<?php

namespace Teknisa\ACL\Controllers;

use Teknisa\ACL\Services\AccessProfile as AccessProfileService;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Teknisa\ACL\Util\AclUtilities;
use Teknisa\Libs\Util\Environment;
use Teknisa\ACL\Util\InstanceProvider;

class AccessProfile extends Crud{

    const IS_NEW = "__is_new";

    protected $dataSourceName = "accessProfile";

    /** @var AccessProfileService */
    protected $accessProfileService;
    
    /** @var Teknisa\Libs\Util\Environment */
    protected $environment;

    public function __construct(Manager $manager, AccessProfileService $accessProfileService, Environment $environment) {
        $this->accessProfileService = $accessProfileService;
        $this->environment = $environment;
        parent::__construct($manager);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions        = $request->getFilterCriteria()->getConditions();
        $showFullOperator  = AclUtilities::getValueInFilterDataSourceByName($conditions, 'SHOW_FULL_OPERATOR', true);
        $newFilterCriteria = AclUtilities::createNewFilterCriteria($conditions, $this->dataSourceName);
        $newFilterCriteria->setPageSize($request->getFilterCriteria()->getPageSize());
        $newFilterCriteria->setPage($request->getFilterCriteria()->getPage());
        $request           = new Request\Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $request->getFilterCriteria()->addCondition(AccessProfileService::NRORG, $this->accessProfileService->getNrOrgFilter());
        $idutlperfilfilial = $this->environment->getIdUtlPerfilFilial();
        if($idutlperfilfilial == 'S' && $showFullOperator) {
            $this->dataSourceName =  "accessProfileWithIdBlockFilial";
        }
        parent::find($request, $response);
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        $labels = $rows[0]['ACL_LABELS_TRADUCTION_LOG'];
        $index = 10;
        $maxNrPerfilAcesso = $this->accessProfileService->getMaxNrPerfilAcesso();
        $rowsWithDefaultValues = array();
        $nmOperador =  $this->accessProfileService->getCurrentUserName();
        foreach($rows as &$row) {
                $showFullOperator  = $row['SHOW_FULL_OPERATOR'];
            if ($row[self::IS_NEW]) {
                $row[AccessProfileService::NRPERFILACESSO] = $maxNrPerfilAcesso+$index;
                $row[AccessProfileService::NMOPERINCLUSAO] = $nmOperador;
                $index += 10;
            }
        if($this->environment->getIdUtlperfilFilial() == 'S' && $showFullOperator){
            $this->dataSourceName = 'accessProfileWithIdBlockFilial';
        }
            $row[AccessProfileService::NMOPERULTATU] = $nmOperador;
            $this->accessProfileService->setDefaultValues($row);
            $rowsWithDefaultValues[] = $row;
        }

        $this->accessProfileService->saveLogLib($rows, $labels);

        parent::save($request, $response);
        $defaultValuesDataSet = new DataSet("accessProfileRowsDefaultValues", $rowsWithDefaultValues);
        $response->addDataSet($defaultValuesDataSet);
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response) {
        $profilesToDelete = array();
        foreach ($request->getDataSet()->getRows() as &$row) {
            $profilesToDelete[] = $row[AccessProfileService::NRPERFILACESSO];
        }
        $this->accessProfileService->deleteDependencies($profilesToDelete);
        parent::delete($request, $response);
    }

    public function deleteRow(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $labels = $row['ACL_LABELS_TRADUCTION_LOG'];
        $profileToDelete = array($row[AccessProfileService::NRPERFILACESSO]);
        $this->accessProfileService->deleteDependencies($profileToDelete, $labels, $row['NMPERFILACESSO']);
        $dataSetToDelete = new DataSet($this->dataSourceName, array($row));
        $deletedRows = $this->dataSourceManager->delete($dataSetToDelete);
        $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        $nrDeletedRows = count($deletedRows);
        $response->addNotification(new DTO\Response\Notification("{$nrDeletedRows} row(s) deleted.", DTO\Response\Notification::TYPE_SUCCESS));
    }
    
    
    public function findAccessControllUtilization(Request $request, Response $response){
        $return = $this->accessProfileService->findAccessControllUtilization();
        $response->addDataSet(new DataSet('findAccessControllUtilization', $return));
    }
    
    public function changeAccessProfile(Request $request, Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $labels = $row['ACL_LABELS_TRADUCTION_LOG'];
        $idutctrlacesweb = $row['IDUTCTRLACESWEB'];
        $return = $this->accessProfileService->changeAccessProfile($idutctrlacesweb, $labels);
        $response->addDataSet(new DataSet('changeAccessProfile', [$return]));
    }

    public function validateActivation(Request $request, Response $response){
        $return = $this->accessProfileService->validateActivation();
        $response->addDataSet(new DataSet('validateActivation', $return));
    }
    
    public function findPerfAcessControl(Request $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $nrAccessProfile = AclUtilities::getValueInFilterDataSourceByName($conditions, 'NRPERFILACESSO');
        $return = $this->accessProfileService->findPerfAcessControl($nrAccessProfile);
        $response->addDataSet(new DataSet('findPerfAcessControl', $return));
    }
    
    public function findPerfAccessByControlProfile(Request $request, Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $nrAccessProfile = AclUtilities::getValueInFilterDataSourceByName($conditions, 'NRPERFILACESSO');
        $return = $this->accessProfileService->findPerfAccessByControlProfile($nrAccessProfile);
        $response->addDataSet(new DataSet('findPerfAcessControl', $return));
    }
    
    public function savePerfAccessByControlProfile(Request $request, Response $response){
        $rows = $request->getDataSet()->getRows();
        $labels = $rows[0]['ACL_LABELS_TRADUCTION_LOG'];
        $return = $this->accessProfileService->savePerfAccessByControlProfile($rows, $labels);
        $response->addDataSet(new DataSet('savePerfAcessControl', $return));
    }
    
    public function deletePerfAccesByControlProfile(Request\DataSet $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows();
            $labels = $row[0]['ACL_LABELS_TRADUCTION_LOG'];
            $nrAccessProfile = $row[0]['NRPERFILACESSO'];
            $nrControlProfile = $row[0]['NRPERFACESSCONT'];
            $profile = $this->accessProfileService->deleteControlProfile($nrAccessProfile, $nrControlProfile, $labels);
        }catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function changeAccessControl(Request $request, Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $labels = $row['ACL_LABELS_TRADUCTION_LOG'];
        $idutlperfilfilial = $row['IDUTLPERFILFILIAL'];
        $return = $this->accessProfileService->changeAccessControl($idutlperfilfilial, $labels);
        $this->environment->setIdUtlPerfilFilial($return['idutlperfilfilial']);
        $response->addDataSet(new DataSet('changeAccessControl', [$return]));
    }

    public function aclZhuAclElementExportRules(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $labels = $row['ACL_LABELS_TRADUCTION_LOG'];
        $this->accessProfileService->deleteLogLibExportRules($row, $labels);
        InstanceProvider::getAcessControlRule()->exportRules($request, $response);
        $this->accessProfileService->insertLogLibExportRules($row, $labels);
    }

    public function aclZhuAclElementRules(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $labels = $row['ACL_LABELS_TRADUCTION_LOG'];
        $modifiedRules = $this->accessProfileService->aclZhuAclElementRules($request, $response, $labels);
        $rulesDataSet = new DataSet('aclZhuAccessControlRule', $modifiedRules);
        $response->addDataSet($rulesDataSet);
    }
}